/*
 * Copyright 2017 The OpenSSL Project Authors. All Rights Reserved.
 * Copyright 2017 Ribose Inc. All Rights Reserved.
 *
 * Licensed under the OpenSSL license (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#ifndef HEADER_SM3_H
# define HEADER_SM3_H

# include <stddef.h>
# include <openssl/opensslconf.h>
# include <openssl/evp.h>

# ifdef OPENSSL_NO_SM3
#  error SM3 is disabled.
# endif

# define SM3_DIGEST_LENGTH 32
# define SM3_WORD unsigned int

# define SM3_CBLOCK      64
# define SM3_LBLOCK      (SM3_CBLOCK/4)

# define SM3_DIGEST_SIZE SM3_DIGEST_LENGTH
# define SM3_BLOCK_SIZE  SM3_CBLOCK

typedef struct SM3state_st {
   SM3_WORD A, B, C, D, E, F, G, H;
   SM3_WORD Nl, Nh;
   SM3_WORD data[SM3_LBLOCK];
   unsigned int num;
} SM3_CTX;

struct sm3_state {
   unsigned int state[SM3_DIGEST_SIZE / 4];
   unsigned long count;
   unsigned char buffer[SM3_BLOCK_SIZE];
};

typedef void (sm3_block_fn)(struct sm3_state *sst, unsigned char const *src, int blocks);

int sm3_init(SM3_CTX *c);
int sm3_update(SM3_CTX *c, const void *data, size_t len);
int sm3_final(unsigned char *md, SM3_CTX *c);

int sm3_base_init(EVP_MD_CTX *ctx);
int sm3_avx_update(EVP_MD_CTX *ctx, const void *data, size_t len);
int sm3_avx_final(EVP_MD_CTX *ctx, unsigned char *out);

#define SM3_IVA 0x7380166fUL
#define SM3_IVB 0x4914b2b9UL
#define SM3_IVC 0x172442d7UL
#define SM3_IVD 0xda8a0600UL
#define SM3_IVE 0xa96f30bcUL
#define SM3_IVF 0x163138aaUL
#define SM3_IVG 0xe38dee4dUL
#define SM3_IVH 0xb0fb0e4eUL

#endif
