/*
 * Copyright 2017 The OpenSSL Project Authors. All Rights Reserved.
 * Copyright 2017 Ribose Inc. All Rights Reserved.
 *
 * Licensed under the OpenSSL license (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#ifndef HEADER_SM4_H
# define HEADER_SM4_H

# include <openssl/opensslconf.h>
# include <openssl/e_os2.h>
# include <openssl/modes.h>

# ifdef OPENSSL_NO_SM4
#  error SM4 is disabled.
# endif

# define SM4_ENCRYPT     1
# define SM4_DECRYPT     0

# define SM4_BLOCK_SIZE    16
# define SM4_KEY_SCHEDULE  32

# define SM4_CRYPT4_BLOCK_SIZE   (4 * SM4_BLOCK_SIZE)
# define SM4_CRYPT8_BLOCK_SIZE   (8 * SM4_BLOCK_SIZE)
# define SM4_CRYPT16_BLOCK_SIZE  (16 * SM4_BLOCK_SIZE)

#define QCHAR(a, b, c, d) ((a) + ((b) << 8) + ((c) << 16) + ((d) << 24))
#define CPUID_IS(a, b, c, ebx, ecx, edx)        \
        (!((ebx ^ (a))|(edx ^ (b))|(ecx ^ (c))))

#define CPUID_HYGON1 QCHAR('H', 'y', 'g', 'o')
#define CPUID_HYGON2 QCHAR('n', 'G', 'e', 'n')
#define CPUID_HYGON3 QCHAR('u', 'i', 'n', 'e')


typedef struct SM4_KEY_st {
    uint32_t rk[SM4_KEY_SCHEDULE];
    uint32_t rk_dec[SM4_KEY_SCHEDULE];
    uint32_t sm4_avx_crypt;
} SM4_KEY;

enum x86_vendor_type {
    X86_VENDOR_HYGON = 9,
    X86_VENDOR_UNKNOWN = 0xFF,
};

int SM4_set_key(const uint8_t *key, SM4_KEY *ks);

void SM4_encrypt(const uint8_t *in, uint8_t *out, const SM4_KEY *ks);

void SM4_decrypt(const uint8_t *in, uint8_t *out, const SM4_KEY *ks);

void sm4_avx_cbc128_decrypt(const unsigned char *in, unsigned char *out,
                           size_t len, const void *key,
                           unsigned char ivec[16], block128_f block);

void sm4_avx_cfb128_decrypt(const unsigned char *in, unsigned char *out,
                           size_t len, const void *key, unsigned char ivec[16],
                           int *num, int enc, block128_f block);

void sm4_avx_ctr128_encrypt(const unsigned char *in, unsigned char *out,
                           size_t len, const void *key, unsigned char ivec[16],
                           unsigned char ecount_buf[16], unsigned int *num,
                           block128_f block);

int sm4_avx_gcm128_encrypt(GCM128_CONTEXT *ctx,
                           const unsigned char *in, unsigned char *out,
                           size_t len);

int sm4_avx_gcm128_decrypt(GCM128_CONTEXT *ctx,
                           const unsigned char *in, unsigned char *out,
                           size_t len);

#endif
